package com.mortals.xhx.common.utils;

import cn.hutool.core.net.url.UrlBuilder;
import cn.hutool.core.util.CharsetUtil;
import cn.hutool.core.util.ReUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.mortals.framework.common.Rest;
import com.mortals.framework.util.DataUtil;
import com.mortals.xhx.common.code.SourceEnum;
import com.mortals.xhx.common.code.YesNoEnum;
import com.mortals.xhx.module.matter.model.MatterEntity;
import lombok.extern.slf4j.Slf4j;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;
import org.springframework.util.ObjectUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 解析事项类型，主题，热度等
 */
@Slf4j
public class MatterTypeHtmlParseUtil {


    /**
     * 获取主题页码极总条数
     *
     * @param params
     * @param url
     * @return
     */
    public static Rest<Map<String, Integer>> getThemeMatterPage(Map<String, String> params, String url) {
        String matterTotalExp = "//input[@id=\"result_countTheme\"]";
        String matterPageExp = "//input[@id=\"pageNumTheme\"]";
        Map<String, Integer> resultMap = new HashMap<>();
        try {
            Document dom = Jsoup.connect(url).data(params).get();
            Elements elements = dom.selectXpath(matterTotalExp);
            if (elements.size() > 0) {
                Integer total = elements.get(0) == null ? 0 : DataUtil.converStr2Int(elements.get(0).attr("value"), 0);
                resultMap.put("total", total);
            } else {
                resultMap.put("total", 0);
            }

            elements = dom.selectXpath(matterPageExp);
            if (elements.size() > 0) {
                Integer pageNum = elements.get(0) == null ? 0 : DataUtil.converStr2Int(elements.get(0).attr("value"), 0);
                resultMap.put("pageNum", pageNum);
            } else {
                resultMap.put("pageNum", 0);
            }

        } catch (Exception e) {
            log.error("获取事项数量异常！params:" + JSON.toJSONString(params), e);
            return Rest.fail(e.getMessage());
        }

        return Rest.ok(resultMap);
    }

    /**
     * 获取主题页码极总条数
     *
     * @param params
     * @param url
     * @return
     */
    public static Rest<Map<String, Integer>> getTownThemeMatterPage(Map<String, String> params, String url) {
        String matterTotalExp = "//input[@id=\"result_count\"]";
        String matterPageExp = "//input[@id=\"pageNum\"]";
        Map<String, Integer> resultMap = new HashMap<>();
        try {
            Document dom = Jsoup.connect(url).data(params).get();
            Elements elements = dom.selectXpath(matterTotalExp);
            if (elements.size() > 0) {
                Integer total = elements.get(0) == null ? 0 : DataUtil.converStr2Int(elements.get(0).attr("value"), 0);
                resultMap.put("total", total);
            } else {
                resultMap.put("total", 0);
            }

            elements = dom.selectXpath(matterPageExp);
            if (elements.size() > 0) {
                Integer pageNum = elements.get(0) == null ? 0 : DataUtil.converStr2Int(elements.get(0).attr("value"), 0);
                resultMap.put("pageNum", pageNum);
            } else {
                resultMap.put("pageNum", 0);
            }

        } catch (Exception e) {
            log.error("获取事项数量异常！params:" + JSON.toJSONString(params), e);
            return Rest.fail(e.getMessage());
        }

        return Rest.ok(resultMap);
    }


    /**
     * 获取主题列表
     *
     * @param params
     * @param url
     * @return
     */
    public static Rest<Map<String, String>> getThemeList(Map<String, String> params, String url) {
        String ThemeListExp = "//ul[@class=\"zhuti_list\"]//li//a";
        Map<String, String> map = new HashMap<>();
        try {
            Document dom = Jsoup.connect(url).data(params).get();
            Elements elements = dom.selectXpath(ThemeListExp);

            for (int i = 0; i < elements.size(); i++) {
                Element element = elements.get(i);
                if (element == null) {
                    continue;
                }
                //主题编码
                String themeCode = element.attr("id");
                String title = element.attr("title");
                map.put(themeCode, title);
            }

        } catch (Exception e) {
            log.error("获取列表异常！params:" + JSON.toJSONString(params), e);
            return Rest.fail(e.getMessage());
        }
        return Rest.ok(map);
    }

    /**
     * 获取乡镇主题列表
     *
     * @param params
     * @param url
     * @return
     */
    public static Rest<Map<String, String>> getTownThemeList(Map<String, String> params, String url) {
        String ThemeListExp = "//ul[@class='fwlx_list cf']//li";
        Map<String, String> map = new HashMap<>();
        try {
            Document dom = Jsoup.connect(url).data(params).get();
            Elements elements = dom.selectXpath(ThemeListExp);

            for (int i = 0; i < elements.size(); i++) {
                Element element = elements.get(i);
                if (element == null) {
                    continue;
                }
                //主题编码
                String title = element.attr("title");
                //changeTheme('005', '1');
                String onclick = element.attr("onclick");
                List<String> list = ReUtil.findAllGroup0("'(.*?)'", onclick);
                if (list.size() > 1) {
                    String themeCode = StrUtil.subBetween(list.get(0), "'", "'");
                    //1 个人服务 2 法人服务
                    String type = StrUtil.subBetween(list.get(1), "'", "'");
                    if (!"".equals(themeCode)) {
                        map.put(themeCode + "&" + type, title);
                    }
                }

            }

        } catch (Exception e) {
            log.error("获取列表异常！params:" + JSON.toJSONString(params), e);
            return Rest.fail(e.getMessage());
        }
        return Rest.ok(map);
    }

    /**
     * 个人服务主题获取事项列表
     *
     * @param params
     * @param url
     * @return
     */
    public static Rest<List<MatterEntity>> getGrThemeMatterList(Map<String, String> params, String url) {
        String matterListExp = "//div[@class=\"sx_list\"]//span[1]";
        String matterListLiExp = "//div[@class=\"sx_list\"]//li/a[1]";
        List<MatterEntity> matterEntityList = new ArrayList<>();
        try {
            Document dom = Jsoup.connect(url).data(params).get();
            Elements elements = dom.selectXpath(matterListExp);

            for (int i = 0; i < elements.size(); i++) {
                Element element = elements.get(i);
                if (element == null) {
                    continue;
                }
                String title = element.attr("title");
                String href = element.firstElementChild().attr("href");
                if (href.equalsIgnoreCase("javascript:void(0)")) {
                    continue;
                }
                String onclick = element.firstElementChild().attr("onclick");
                href = StrUtil.subBetween(onclick, "ywblurl('", "',");
                if (ObjectUtils.isEmpty(href)) {
                    continue;
                }
                buildMatter(matterEntityList, title, href);
            }

            elements = dom.selectXpath(matterListLiExp);
            for (int i = 0; i < elements.size(); i++) {
                Element element = elements.get(i);
                if (element == null) {
                    continue;
                }
                String title = element.attr("title");
                // String href = element.attr("href");
                String onclick = element.attr("onclick");
                String href = StrUtil.subBetween(onclick, "ywblurl('", "',");
                if (ObjectUtils.isEmpty(href)) {
                    System.out.println("子项 href is empty!,dom:" + dom.html());
                    continue;
                }
                buildMatter(matterEntityList, title, href);
            }

        } catch (Exception e) {
            log.error("获取列表异常！params:" + JSON.toJSONString(params), e);
            return Rest.fail(e.getMessage());
        }

        return Rest.ok(matterEntityList);
    }

    public static Rest<List<MatterEntity>> getFrThemeMatterList(Map<String, String> params, String url) {
        String matterListExp = "//div[@class=\"sx_list\"]//span[1]";
        String matterListLiExp = "//div[@class=\"sx_list\"]//li/a[1]";
        List<MatterEntity> matterEntityList = new ArrayList<>();
        try {
            Document dom = Jsoup.connect(url).data(params).get();
            Elements elements = dom.selectXpath(matterListExp);

            for (int i = 0; i < elements.size(); i++) {
                Element element = elements.get(i);
                if (element == null) {
                    continue;
                }
                String title = element.attr("title");
                String onclick = element.attr("onclick");
                String href = StrUtil.subBetween(onclick, "ywblurl('", "',");
                if (ObjectUtils.isEmpty(href)) {
                    continue;
                }
                buildMatter(matterEntityList, title, href);
            }

            elements = dom.selectXpath(matterListLiExp);
            for (int i = 0; i < elements.size(); i++) {
                Element element = elements.get(i);
                if (element == null) {
                    continue;
                }
                String title = element.attr("title");
                String onclick = element.attr("onclick");
                String href = StrUtil.subBetween(onclick, "ywblurl('", "',");
                if (ObjectUtils.isEmpty(href)) {
                    log.info("子项 href is empty!,dom:" + element.html());
                    continue;
                }
                buildMatter(matterEntityList, title, href);
            }

        } catch (Exception e) {
            log.error("获取列表异常！params:" + JSON.toJSONString(params), e);
            return Rest.fail(e.getMessage());
        }

        return Rest.ok(matterEntityList);
    }


    /**
     * 乡镇服务主题获取事项列表
     *
     * @param params
     * @param url
     * @return
     */
    public static Rest<List<MatterEntity>> getTownThemeMatterList(Map<String, String> params, String url) {
        String matterListExp = "//div[@class=\"sx_list\"]//span[1]";
        String matterListLiExp = "//div[@class=\"sx_list\"]//li/a[1]";
        List<MatterEntity> matterEntityList = new ArrayList<>();
        try {
            Document dom = Jsoup.connect(url).data(params).get();
            Elements elements = dom.selectXpath(matterListExp);

            for (int i = 0; i < elements.size(); i++) {
                Element element = elements.get(i);
                if (element == null) {
                    continue;
                }
                String title = element.attr("title");
                String href = element.firstElementChild().attr("href");
                if (href.equalsIgnoreCase("javascript:void(0)")) {
                    continue;
                }
/*                String onclick = element.firstElementChild().attr("onclick");
                href = StrUtil.subBetween(onclick, "ywblurl('", "',");
                if (ObjectUtils.isEmpty(href)) {
                    continue;
                }*/
                buildMatter(matterEntityList, title, href);
            }

            elements = dom.selectXpath(matterListLiExp);
            for (int i = 0; i < elements.size(); i++) {
                Element element = elements.get(i);
                if (element == null) {
                    continue;
                }
                String title = element.attr("title");
                String href = element.attr("href");
      /*          String onclick = element.attr("onclick");
                String href = StrUtil.subBetween(onclick, "ywblurl('", "',");
                if (ObjectUtils.isEmpty(href)) {
                    System.out.println("子项 href is empty!,dom:" + dom.html());
                    continue;
                }*/
                buildMatter(matterEntityList, title, href);
            }

        } catch (Exception e) {
            log.error("获取列表异常！params:" + JSON.toJSONString(params), e);
            return Rest.fail(e.getMessage());
        }

        return Rest.ok(matterEntityList);
    }

    private static void buildMatter(List<MatterEntity> matterEntityList, String title, String href) {
        UrlBuilder builder = UrlBuilder.ofHttp(href, CharsetUtil.CHARSET_UTF_8);
        String itemCode = builder.getQuery().get("itemCode").toString();
        String taskType = builder.getQuery().get("taskType").toString();
        String deptCode = builder.getQuery().get("deptCode").toString();
        String areaCode = builder.getQuery().get("areaCode").toString();

        MatterEntity matterEntity = new MatterEntity();
        matterEntity.initAttrValue();
        matterEntity.setTcode(itemCode);
        matterEntity.setTname(title);
        matterEntity.setDeptCode(deptCode);
        matterEntity.setMatterNo(itemCode);
        matterEntity.setAreaCode(areaCode);
        matterEntity.setMatterName(title);
        matterEntity.setUrl(href);
        matterEntity.setSource(SourceEnum.政务网.getValue());
        matterEntityList.add(matterEntity);
    }


    public static void main(String[] args) {
        //个人主题服务
      /*  String url = "http://www.sczwfw.gov.cn/jiq/front/item/gr_index";
        HashMap<String, String> params = new HashMap<>();
        params.put("areaCode", "510107000000");
        Rest<Map<String, String>> grThemeRest = MatterTypeHtmlParseUtil.getThemeList(params, url);
        System.out.println(JSON.toJSONString(grThemeRest));

        //法人服务
        url = "http://www.sczwfw.gov.cn/jiq/front/item/fr_index";
        params = new HashMap<>();
        params.put("areaCode", "510500000000");
        Rest<Map<String, String>> frThemeRest = MatterTypeHtmlParseUtil.getThemeList(params, url);
        System.out.println(JSON.toJSONString(frThemeRest));

        HashMap<String, String> params1 = new HashMap<>();
        params1.put("dxType", "2");
        params1.put("areaCode", "510500000000");
        params1.put("deptCode", "");
        params1.put("searchtext", "");
        params1.put("taskType", "");
        params1.put("type", "1");


        List<MatterEntity> allList = new ArrayList<>();
        grThemeRest.getData().entrySet().stream().forEach(item -> {
            params1.put("theme", item.getKey());
            String url1 = "http://www.sczwfw.gov.cn/jiq/interface/item/tags";
            Rest<Map<String, Integer>> themeMatterPageRest = MatterTypeHtmlParseUtil.getThemeMatterPage(params1, url1);
            //System.out.println("themePageRest:"+JSON.toJSONString(themeMatterPageRest));
            Integer pageNum = themeMatterPageRest.getData().get("pageNum");
            Integer total = themeMatterPageRest.getData().get("total");
            if (pageNum > 0 && total > 0) {
                int count = 0;
                for (int i = 1; i <= pageNum; i++) {
                    params1.put("pageno", String.valueOf(i));
                    Rest<List<MatterEntity>> restList = MatterTypeHtmlParseUtil.getGrThemeMatterList(params1, url1);
                    if (restList.getCode() == YesNoEnum.YES.getValue()) {
                        allList.addAll(restList.getData());
                        count += restList.getData().size();
                    }
                }
                if (count != total) {
                    log.info(String.format("获取数量异常,count:%s,total:%s,theme:%s", count, total, item.getKey()));
                }
            }
        });*/


        //System.out.println(JSON.toJSONString(allList));

/*        for (int i = 1; i <= rest.getData().get("pageNum"); i++) {
            params1.put("pageno", String.valueOf(i));
            Rest<List<MatterEntity>> restList = MatterTypeHtmlParseUtil.getMatterList(params1, url1);
            if (restList.getCode() == YesNoEnum.YES.getValue() && !ObjectUtils.isEmpty(restList.getData())) {
                allList.addAll(restList.getData());
            }
        }*/


        //http://www.sczwfw.gov.cn/jiq/front/item/town?areaCode=510105521000

        //村镇主题服务
        String url3 = "http://www.sczwfw.gov.cn/jiq/front/item/town";
        Map params3 = new HashMap<>();
        params3.put("areaCode", "510105521000");
        Rest<Map<String, String>> townThemeRest = MatterTypeHtmlParseUtil.getTownThemeList(params3, url3);
        //System.out.println(JSON.toJSONString(townThemeRest));
        HashMap<String, String> params1 = new HashMap<>();
        params1.put("dxType", "54");
        params1.put("areaCode", "510105521000");
        params1.put("type", "1");
        List<MatterEntity> allList = new ArrayList<>();
        townThemeRest.getData().entrySet().stream().forEach(item -> {
            String[] vals = item.getKey().split("&");
            params1.put("theme", vals[0]);
            params1.put("userType", vals[1]);
            //http://www.sczwfw.gov.cn/jiq/interface/item/tags?dxType=54&areaCode=510105521000&theme=085&userType=1&pageno=2
            String url1 = "http://www.sczwfw.gov.cn/jiq/interface/item/tags";
            Rest<Map<String, Integer>> themeMatterPageRest = MatterTypeHtmlParseUtil.getTownThemeMatterPage(params1, url1);
            //System.out.println("themePageRest:"+JSON.toJSONString(themeMatterPageRest));
            Integer pageNum = themeMatterPageRest.getData().get("pageNum");
            Integer total = themeMatterPageRest.getData().get("total");
            if (pageNum > 0 && total > 0) {
                int count = 0;
                for (int i = 1; i <= pageNum; i++) {
                    params1.put("pageno", String.valueOf(i));
                    Rest<List<MatterEntity>> restList = MatterTypeHtmlParseUtil.getTownThemeMatterList(params1, url1);
                    if (restList.getCode() == YesNoEnum.YES.getValue()) {
                        allList.addAll(restList.getData());
                        count += restList.getData().size();
                    }
                }
                if (count != total) {
                    log.info(String.format("获取数量异常,count:%s,total:%s,theme:%s", count, total, item.getKey()));
                }
            }
        });

    }

}
